@@ -1,5 +1,5 @@
 /* gpgkeys_ldap.c - talk to a LDAP keyserver
- * Copyright (C) 2001, 2002, 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2004, 2005, 2006 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -33,6 +34,11 @@
 #include <winsock2.h>
 #include <winldap.h>
 #else
+#ifdef NEED_LBER_H
+#include <lber.h>
+#endif
+/* For OpenLDAP, to enable the API that we're using. */
+#define LDAP_DEPRECATED 1
 #include <ldap.h>
 #endif
 
@@ -47,30 +53,17 @@
 extern char *optarg;
 extern int optind;
 
-#define GET    0
-#define SEND   1
-#define SEARCH 2
-#define MAX_LINE 256
-
-static int verbose=0,include_disabled=0,include_revoked=0,include_subkeys=0;
 static int real_ldap=0;
 static char *basekeyspacedn=NULL;
-static char host[80]={'\0'};
-static char portstr[10]={'\0'};
 static char *pgpkeystr="pgpKey";
 static FILE *input=NULL,*output=NULL,*console=NULL;
 static LDAP *ldap=NULL;
+static struct ks_options *opt;
 
 #ifndef HAVE_TIMEGM
 time_t timegm(struct tm *tm);
 #endif
 
-struct keylist
-{
-  char str[MAX_LINE];
-  struct keylist *next;
-};
-
 static int
 ldap_err_to_gpg_err(int err)
 {
@@ -257,7 +250,7 @@ join_two_modlists(LDAPMod ***one,LDAPMod **two)
    the attribute in question exists or not. */
 
 static int
-make_one_attr(LDAPMod ***modlist,int unique,char *attr,const char *value)
+make_one_attr(LDAPMod ***modlist,char *attr,const char *value)
 {
   LDAPMod **m;
   int nummods=0;
@@ -279,7 +272,8 @@ make_one_attr(LDAPMod ***modlist,int unique,char *attr,const char *value)
 	  if(ptr)
 	    for(ptr=(*m)->mod_values;*ptr;ptr++)
 	      {
-		if(unique && strcmp(*ptr,value)==0)
+		/* Duplicate value */
+		if(strcmp(*ptr,value)==0)
 		  return 1;
 		numvalues++;
 	      }
@@ -372,8 +366,8 @@ build_attrs(LDAPMod ***modlist,char *line)
 
       if(strlen(tok)==16)
 	{
-	  make_one_attr(modlist,0,"pgpCertID",tok);
-	  make_one_attr(modlist,0,"pgpKeyID",&tok[8]);
+	  make_one_attr(modlist,"pgpCertID",tok);
+	  make_one_attr(modlist,"pgpKeyID",&tok[8]);
 	}
       else
 	return;
@@ -385,11 +379,11 @@ build_attrs(LDAPMod ***modlist,char *line)
       switch(atoi(tok))
 	{
 	case 1:
-	  make_one_attr(modlist,0,"pgpKeyType","RSA");
+	  make_one_attr(modlist,"pgpKeyType","RSA");
 	  break;
 
 	case 17:
-	  make_one_attr(modlist,0,"pgpKeyType","DSS/DH");
+	  make_one_attr(modlist,"pgpKeyType","DSS/DH");
 	  break;
 	}
 
@@ -407,7 +401,7 @@ build_attrs(LDAPMod ***modlist,char *line)
 	  if(val<99999 && val>0)
 	    {
 	      sprintf(padded,"%05u",atoi(tok));
-	      make_one_attr(modlist,0,"pgpKeySize",padded);
+	      make_one_attr(modlist,"pgpKeySize",padded);
 	    }
 	}
 
@@ -420,7 +414,7 @@ build_attrs(LDAPMod ***modlist,char *line)
 	  char *stamp=epoch2ldaptime(atoi(tok));
 	  if(stamp)
 	    {
-	      make_one_attr(modlist,0,"pgpKeyCreateTime",stamp);
+	      make_one_attr(modlist,"pgpKeyCreateTime",stamp);
 	      free(stamp);
 	    }
 	}
@@ -434,7 +428,7 @@ build_attrs(LDAPMod ***modlist,char *line)
 	  char *stamp=epoch2ldaptime(atoi(tok));
 	  if(stamp)
 	    {
-	      make_one_attr(modlist,0,"pgpKeyExpireTime",stamp);
+	      make_one_attr(modlist,"pgpKeyExpireTime",stamp);
 	      free(stamp);
 	    }
 	}
@@ -464,8 +458,8 @@ build_attrs(LDAPMod ***modlist,char *line)
 	"(&(pgpUserID=*isabella*)(pgpDisabled=0))"
       */
 
-      make_one_attr(modlist,0,"pgpDisabled",disabled?"1":"0");
-      make_one_attr(modlist,0,"pgpRevoked",revoked?"1":"0");
+      make_one_attr(modlist,"pgpDisabled",disabled?"1":"0");
+      make_one_attr(modlist,"pgpRevoked",revoked?"1":"0");
     }
   else if(ascii_strcasecmp("sub",record)==0)
     {
@@ -476,7 +470,7 @@ build_attrs(LDAPMod ***modlist,char *line)
 	return;
 
       if(strlen(tok)==16)
-	make_one_attr(modlist,0,"pgpSubKeyID",tok);
+	make_one_attr(modlist,"pgpSubKeyID",tok);
       else
 	return;
 
@@ -498,7 +492,7 @@ build_attrs(LDAPMod ***modlist,char *line)
 	  if(val<99999 && val>0)
 	    {
 	      sprintf(padded,"%05u",atoi(tok));
-	      make_one_attr(modlist,0,"pgpKeySize",padded);
+	      make_one_attr(modlist,"pgpKeySize",padded);
 	    }
 	}
 
@@ -540,7 +534,7 @@ build_attrs(LDAPMod ***modlist,char *line)
       /* We don't care about the other info provided in the uid: line
 	 since the LDAP schema doesn't need it. */
 
-      make_one_attr(modlist,0,"pgpUserID",userid);
+      make_one_attr(modlist,"pgpUserID",userid);
     }
   else if(ascii_strcasecmp("sig",record)==0)
     {
@@ -550,7 +544,7 @@ build_attrs(LDAPMod ***modlist,char *line)
 	return;
 
       if(strlen(tok)==16)
-	make_one_attr(modlist,1,"pgpSignerID",tok);
+	make_one_attr(modlist,"pgpSignerID",tok);
     }
 }
 
@@ -599,17 +593,17 @@ send_key(int *eof)
   /* Start by nulling out all attributes.  We try and do a modify
      operation first, so this ensures that we don't leave old
      attributes lying around. */
-  make_one_attr(&modlist,0,"pgpDisabled",NULL);
-  make_one_attr(&modlist,0,"pgpKeyID",NULL);
-  make_one_attr(&modlist,0,"pgpKeyType",NULL);
-  make_one_attr(&modlist,0,"pgpUserID",NULL);
-  make_one_attr(&modlist,0,"pgpKeyCreateTime",NULL);
-  make_one_attr(&modlist,0,"pgpSignerID",NULL);
-  make_one_attr(&modlist,0,"pgpRevoked",NULL);
-  make_one_attr(&modlist,0,"pgpSubKeyID",NULL);
-  make_one_attr(&modlist,0,"pgpKeySize",NULL);
-  make_one_attr(&modlist,0,"pgpKeyExpireTime",NULL);
-  make_one_attr(&modlist,0,"pgpCertID",NULL);
+  make_one_attr(&modlist,"pgpDisabled",NULL);
+  make_one_attr(&modlist,"pgpKeyID",NULL);
+  make_one_attr(&modlist,"pgpKeyType",NULL);
+  make_one_attr(&modlist,"pgpUserID",NULL);
+  make_one_attr(&modlist,"pgpKeyCreateTime",NULL);
+  make_one_attr(&modlist,"pgpSignerID",NULL);
+  make_one_attr(&modlist,"pgpRevoked",NULL);
+  make_one_attr(&modlist,"pgpSubKeyID",NULL);
+  make_one_attr(&modlist,"pgpKeySize",NULL);
+  make_one_attr(&modlist,"pgpKeyExpireTime",NULL);
+  make_one_attr(&modlist,"pgpCertID",NULL);
 
   /* Assemble the INFO stuff into LDAP attributes */
 
@@ -728,8 +722,8 @@ send_key(int *eof)
       goto fail;
     }
 
-  make_one_attr(&addlist,0,"objectClass","pgpKeyInfo");
-  make_one_attr(&addlist,0,"pgpKey",key);
+  make_one_attr(&addlist,"objectClass","pgpKeyInfo");
+  make_one_attr(&addlist,"pgpKey",key);
 
   /* Now append addlist onto modlist */
   if(!join_two_modlists(&modlist,addlist))
@@ -1004,7 +998,7 @@ get_key(char *getkey)
       /* fingerprint.  Take the last 16 characters and treat it like a
          long key id */
 
-      if(include_subkeys)
+      if(opt->flags.include_subkeys)
 	sprintf(search,"(|(pgpcertid=%.16s)(pgpsubkeyid=%.16s))",
 		offset,offset);
       else
@@ -1014,7 +1008,7 @@ get_key(char *getkey)
     {
       /* long key id */
 
-      if(include_subkeys)
+      if(opt->flags.include_subkeys)
 	sprintf(search,"(|(pgpcertid=%.16s)(pgpsubkeyid=%.16s))",
 		getkey,getkey);
       else
@@ -1027,10 +1021,10 @@ get_key(char *getkey)
       sprintf(search,"(pgpkeyid=%.8s)",getkey);
     }
 
-  if(verbose>2)
+  if(opt->verbose>2)
     fprintf(console,"gpgkeys: LDAP fetch for: %s\n",search);
 
-  if(!verbose)
+  if(!opt->verbose)
     attrs[2]=NULL; /* keep only pgpkey(v2) and pgpcertid */
 
   err=ldap_search_s(ldap,basekeyspacedn,
@@ -1102,7 +1096,8 @@ get_key(char *getkey)
 		    }
 		  else
 		    {
-		      fprintf(output,"%sKEY 0x%s END\n",vals[0],getkey);
+		      print_nocr(output,vals[0]);
+		      fprintf(output,"\nKEY 0x%s END\n",getkey);
 
 		      ldap_value_free(vals);
 		    }
@@ -1124,6 +1119,139 @@ get_key(char *getkey)
   return ret;
 }
 
+#define LDAP_ESCAPE_CHARS "*()\\"
+
+/* Append string to buffer in a LDAP-quoted way */
+static void
+ldap_quote(char *buffer,const char *string)
+{
+  /* Find the end of buffer */
+  buffer+=strlen(buffer);
+
+  for(;*string;string++)
+    {
+      if(strchr(LDAP_ESCAPE_CHARS,*string))
+	{
+	  sprintf(buffer,"\\%02X",*string);
+	  buffer+=3;
+	}
+      else
+	*buffer++=*string;
+    }
+
+  *buffer='\0';
+}
+
+/* Note that key-not-found is not a fatal error */
+static int
+get_name(char *getkey)
+{
+  LDAPMessage *res,*each;
+  int ret=KEYSERVER_INTERNAL_ERROR,err,count;
+  /* The maximum size of the search, including the optional stuff and
+     the trailing \0 */
+  char search[2+12+(MAX_LINE*3)+2+15+14+1+1+20];
+  /* This ordering is significant - specifically, "pgpcertid" needs to
+     be the second item in the list, since everything after it may be
+     discarded if the user isn't in verbose mode. */
+  char *attrs[]={"replaceme","pgpcertid","pgpuserid","pgpkeyid","pgprevoked",
+		 "pgpdisabled","pgpkeycreatetime","modifytimestamp",
+		 "pgpkeysize","pgpkeytype",NULL};
+  attrs[0]=pgpkeystr; /* Some compilers don't like using variables as
+                         array initializers. */
+
+  /* Build the search string */
+
+  search[0]='\0';
+
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,"(&");
+
+  strcat(search,"(pgpUserID=*");
+  ldap_quote(search,getkey);
+  strcat(search,"*)");
+
+  if(!opt->flags.include_disabled)
+    strcat(search,"(pgpDisabled=0)");
+
+  if(!opt->flags.include_revoked)
+    strcat(search,"(pgpRevoked=0)");
+
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,")");
+
+  if(opt->verbose>2)
+    fprintf(console,"gpgkeys: LDAP fetch for: %s\n",search);
+
+  if(!opt->verbose)
+    attrs[2]=NULL; /* keep only pgpkey(v2) and pgpcertid */
+
+  err=ldap_search_s(ldap,basekeyspacedn,
+		    LDAP_SCOPE_SUBTREE,search,attrs,0,&res);
+  if(err!=0)
+    {
+      int errtag=ldap_err_to_gpg_err(err);
+
+      fprintf(console,"gpgkeys: LDAP search error: %s\n",ldap_err2string(err));
+      fprintf(output,"NAME %s BEGIN\n",getkey);
+      fprintf(output,"NAME %s FAILED %d\n",getkey,errtag);
+      return errtag;
+    }
+
+  count=ldap_count_entries(ldap,res);
+  if(count<1)
+    {
+      fprintf(console,"gpgkeys: key %s not found on keyserver\n",getkey);
+      fprintf(output,"NAME %s BEGIN\n",getkey);
+      fprintf(output,"NAME %s FAILED %d\n",getkey,KEYSERVER_KEY_NOT_FOUND);
+    }
+  else
+    {
+      /* There may be more than one result, but we return them all. */
+
+      each=ldap_first_entry(ldap,res);
+      while(each!=NULL)
+	{
+	  char **vals,**certid;
+
+	  certid=ldap_get_values(ldap,each,"pgpcertid");
+	  if(certid!=NULL)
+	    {
+	      build_info(certid[0],each);
+
+	      fprintf(output,"NAME %s BEGIN\n",getkey);
+
+	      vals=ldap_get_values(ldap,each,pgpkeystr);
+	      if(vals==NULL)
+		{
+		  int errtag=ldap_to_gpg_err(ldap);
+
+		  fprintf(console,"gpgkeys: unable to retrieve key %s "
+			  "from keyserver\n",getkey);
+		  fprintf(output,"NAME %s FAILED %d\n",getkey,errtag);
+		}
+	      else
+		{
+		  print_nocr(output,vals[0]);
+		  fprintf(output,"\nNAME %s END\n",getkey);
+
+		  ldap_value_free(vals);
+		}
+
+	      ldap_value_free(certid);
+	    }
+
+	  each=ldap_next_entry(ldap,each);
+	}
+    }
+
+  ret=KEYSERVER_OK;
+
+  ldap_msgfree(res);
+
+  return ret;
+}
+
 static void
 printquoted(FILE *stream,char *string,char delim)
 {
@@ -1141,7 +1269,7 @@ printquoted(FILE *stream,char *string,char delim)
 /* Returns 0 on success and -1 on error.  Note that key-not-found is
    not an error! */
 static int
-search_key(char *searchkey)
+search_key(const char *searchkey)
 {
   char **vals;
   LDAPMessage *res,*each;
@@ -1149,23 +1277,100 @@ search_key(char *searchkey)
   struct keylist *dupelist=NULL;
   /* The maximum size of the search, including the optional stuff and
      the trailing \0 */
-  char search[2+12+MAX_LINE+2+15+14+1+1];
+  char search[2+1+9+1+3+(MAX_LINE*3)+3+1+15+14+1+1+20];
   char *attrs[]={"pgpcertid","pgpuserid","pgprevoked","pgpdisabled",
 		 "pgpkeycreatetime","pgpkeyexpiretime","modifytimestamp",
 		 "pgpkeysize","pgpkeytype",NULL};
+  enum ks_search_type search_type;
 
   fprintf(output,"SEARCH %s BEGIN\n",searchkey);
 
+  search_type=classify_ks_search(&searchkey);
+
+  if(opt->debug)
+    fprintf(console,"search type is %d, and key is \"%s\"\n",
+	    search_type,searchkey);
+
   /* Build the search string */
 
-  sprintf(search,"%s(pgpuserid=*%s*)%s%s%s",
-	  (!(include_disabled&&include_revoked))?"(&":"",
-	  searchkey,
-	  include_disabled?"":"(pgpdisabled=0)",
-	  include_revoked?"":"(pgprevoked=0)",
-	  !(include_disabled&&include_revoked)?")":"");
+  search[0]='\0';
 
-  if(verbose>2)
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,"(&");
+
+  strcat(search,"(");
+
+  switch(search_type)
+    {
+    case KS_SEARCH_KEYID_SHORT:
+      strcat(search,"pgpKeyID");
+      break;
+
+    case KS_SEARCH_KEYID_LONG:
+      strcat(search,"pgpCertID");
+      break;
+
+    default:
+      strcat(search,"pgpUserID");
+      break;
+    }
+
+  strcat(search,"=");
+
+  switch(search_type)
+    {
+    case KS_SEARCH_SUBSTR:
+      strcat(search,"*");
+      break;
+
+    case KS_SEARCH_MAIL:
+      strcat(search,"*<");
+      break;
+
+    case KS_SEARCH_MAILSUB:
+      strcat(search,"*<*");
+      break;
+
+    case KS_SEARCH_EXACT:
+    case KS_SEARCH_KEYID_LONG:
+    case KS_SEARCH_KEYID_SHORT:
+      break;
+    }
+
+  ldap_quote(search,searchkey);
+
+  switch(search_type)
+    {
+    case KS_SEARCH_SUBSTR:
+      strcat(search,"*");
+      break;
+
+    case KS_SEARCH_MAIL:
+      strcat(search,">*");
+      break;
+
+    case KS_SEARCH_MAILSUB:
+      strcat(search,"*>*");
+      break;
+
+    case KS_SEARCH_EXACT:
+    case KS_SEARCH_KEYID_LONG:
+    case KS_SEARCH_KEYID_SHORT:
+      break;
+    }
+
+  strcat(search,")");
+
+  if(!opt->flags.include_disabled)
+    strcat(search,"(pgpDisabled=0)");
+
+  if(!opt->flags.include_revoked)
+    strcat(search,"(pgpRevoked=0)");
+
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,")");
+
+  if(opt->verbose>2)
     fprintf(console,"gpgkeys: LDAP search for: %s\n",search);
 
   err=ldap_search_s(ldap,basekeyspacedn,
@@ -1206,7 +1411,14 @@ search_key(char *searchkey)
     }
 
   if(err==LDAP_SIZELIMIT_EXCEEDED)
-    fprintf(console,"gpgkeys: search results exceeded server limit.  First %d results shown.\n",count);
+    {
+      if(count==1)
+	fprintf(console,"gpgkeys: search results exceeded server limit."
+		"  First %d result shown.\n",count);
+      else
+	fprintf(console,"gpgkeys: search results exceeded server limit."
+		"  First %d results shown.\n",count);
+    }
 
   free_keylist(dupelist);
   dupelist=NULL;
@@ -1370,12 +1582,12 @@ search_key(char *searchkey)
 }
 
 static void
-fail_all(struct keylist *keylist,int action,int err)
+fail_all(struct keylist *keylist,int err)
 {
   if(!keylist)
     return;
 
-  if(action==SEARCH)
+  if(opt->action==KS_SEARCH)
     {
       fprintf(output,"SEARCH ");
       while(keylist)
@@ -1448,7 +1660,7 @@ find_basekeyspacedn(void)
 		  ldap_value_free(vals);
 		}
 
-	      if(verbose>1)
+	      if(opt->verbose>1)
 		{
 		  vals=ldap_get_values(ldap,si_res,"pgpSoftware");
 		  if(vals)
@@ -1499,7 +1711,7 @@ find_basekeyspacedn(void)
 	  ldap_value_free(vals);
 	}
 
-      if(verbose>1)
+      if(opt->verbose>1)
 	{
 	  vals=ldap_get_values(ldap,si_res,"software");
 	  if(vals)
@@ -1512,7 +1724,7 @@ find_basekeyspacedn(void)
       vals=ldap_get_values(ldap,si_res,"version");
       if(vals)
 	{
-	  if(verbose>1)
+	  if(opt->verbose>1)
 	    fprintf(console,"Version:\t%s\n",vals[0]);
 
 	  /* If the version is high enough, use the new pgpKeyV2
@@ -1543,11 +1755,10 @@ show_help (FILE *fp)
 int
 main(int argc,char *argv[])
 {
-  int debug=0,port=0,arg,err,action=-1,ret=KEYSERVER_INTERNAL_ERROR;
-  char line[MAX_LINE];
-  int version,failed=0,use_ssl=0,use_tls=0,bound=0;
+  int port=0,arg,err,ret=KEYSERVER_INTERNAL_ERROR;
+  char line[MAX_LINE],*binddn=NULL,*bindpw=NULL;
+  int failed=0,use_ssl=0,use_tls=0,bound=0;
   struct keylist *keylist=NULL,*keyptr=NULL;
-  unsigned int timeout=DEFAULT_KEYSERVER_TIMEOUT;
 
   console=stderr;
 
@@ -1587,7 +1798,6 @@ main(int argc,char *argv[])
 	break;
       }
 
-
   if(argc>optind)
     {
       input=fopen(argv[optind],"r");
@@ -1605,76 +1815,34 @@ main(int argc,char *argv[])
   if(output==NULL)
     output=stdout;
 
+  opt=init_ks_options();
+  if(!opt)
+    return KEYSERVER_NO_MEMORY;
+
   /* Get the command and info block */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
     {
-      char commandstr[7];
-      char optionstr[256];
-      char schemestr[80];
-      char hash;
+      char optionstr[MAX_OPTION+1];
 
       if(line[0]=='\n')
 	break;
 
-      if(sscanf(line,"%c",&hash)==1 && hash=='#')
+      err=parse_ks_options(line,opt);
+      if(err>0)
+	{
+	  ret=err;
+	  goto fail;
+	}
+      else if(err==0)
 	continue;
 
-      if(sscanf(line,"COMMAND %6s\n",commandstr)==1)
-	{
-	  commandstr[6]='\0';
-
-	  if(strcasecmp(commandstr,"get")==0)
-	    action=GET;
-	  else if(strcasecmp(commandstr,"send")==0)
-	    action=SEND;
-	  else if(strcasecmp(commandstr,"search")==0)
-	    action=SEARCH;
-
-	  continue;
-	}
-
-      if(sscanf(line,"HOST %79s\n",host)==1)
-	{
-	  host[79]='\0';
-	  continue;
-	}
-
-      if(sscanf(line,"PORT %9s\n",portstr)==1)
-	{
-	  portstr[9]='\0';
-	  port=atoi(portstr);
-	  continue;
-	}
-
-      if(sscanf(line,"SCHEME %79s\n",schemestr)==1)
-	{
-	  schemestr[79]='\0';
-	  if(strcasecmp(schemestr,"ldaps")==0)
-	    {
-	      port=636;
-	      use_ssl=1;
-	    }
-	  continue;
-	}
-
-      if(sscanf(line,"VERSION %d\n",&version)==1)
-	{
-	  if(version!=KEYSERVER_PROTO_VERSION)
-	    {
-	      ret=KEYSERVER_VERSION_ERROR;
-	      goto fail;
-	    }
-
-	  continue;
-	}
-
-      if(sscanf(line,"OPTION %255[^\n]\n",optionstr)==1)
+      if(sscanf(line,"OPTION %" MKSTRING(MAX_OPTION) "[^\n]\n",optionstr)==1)
 	{
 	  int no=0;
 	  char *start=&optionstr[0];
 
-	  optionstr[255]='\0';
+	  optionstr[MAX_OPTION]='\0';
 
 	  if(strncasecmp(optionstr,"no-",3)==0)
 	    {
@@ -1682,35 +1850,7 @@ main(int argc,char *argv[])
 	      start=&optionstr[3];
 	    }
 
-	  if(strcasecmp(start,"verbose")==0)
-	    {
-	      if(no)
-		verbose--;
-	      else
-		verbose++;
-	    }
-	  else if(strcasecmp(start,"include-disabled")==0)
-	    {
-	      if(no)
-		include_disabled=0;
-	      else
-		include_disabled=1;
-	    }
-	  else if(strcasecmp(start,"include-revoked")==0)
-	    {
-	      if(no)
-		include_revoked=0;
-	      else
-		include_revoked=1;
-	    }
-	  else if(strcasecmp(start,"include-subkeys")==0)
-	    {
-	      if(no)
-		include_subkeys=0;
-	      else
-		include_subkeys=1;
-	    }
-	  else if(strncasecmp(start,"tls",3)==0)
+	  if(strncasecmp(start,"tls",3)==0)
 	    {
 	      if(no)
 		use_tls=0;
@@ -1730,13 +1870,6 @@ main(int argc,char *argv[])
 	      else if(start[3]=='\0')
 		use_tls=1;
 	    }
-	  else if(strncasecmp(start,"debug",5)==0)
-	    {
-	      if(no)
-		debug=0;
-	      else if(start[5]=='=')
-		debug=atoi(&start[6]);
-	    }
 	  else if(strncasecmp(start,"basedn",6)==0)
 	    {
 	      if(no)
@@ -1759,24 +1892,98 @@ main(int argc,char *argv[])
 		  real_ldap=1;
 		}
 	    }
-	  else if(strncasecmp(start,"timeout",7)==0)
+	  else if(strncasecmp(start,"binddn",6)==0)
 	    {
 	      if(no)
-		timeout=0;
-	      else
-		timeout=atoi(&start[8]);
+		{
+		  free(binddn);
+		  binddn=NULL;
+		}
+	      else if(start[6]=='=')
+		{
+		  free(binddn);
+		  binddn=strdup(&start[7]);
+		  if(!binddn)
+		    {
+		      fprintf(console,"gpgkeys: out of memory while creating "
+			      "bind DN\n");
+		      ret=KEYSERVER_NO_MEMORY;
+		      goto fail;
+		    }
+
+		  real_ldap=1;
+		}
+	    }
+	  else if(strncasecmp(start,"bindpw",6)==0)
+	    {
+	      if(no)
+		{
+		  free(bindpw);
+		  bindpw=NULL;
+		}
+	      else if(start[6]=='=')
+		{
+		  free(bindpw);
+		  bindpw=strdup(&start[7]);
+		  if(!bindpw)
+		    {
+		      fprintf(console,"gpgkeys: out of memory while creating "
+			      "bind password\n");
+		      ret=KEYSERVER_NO_MEMORY;
+		      goto fail;
+		    }
+
+		  real_ldap=1;
+		}
 	    }
 
 	  continue;
 	}
     }
 
-  if(timeout && register_timeout()==-1)
+  if(!opt->scheme)
+    {
+      fprintf(console,"gpgkeys: no scheme supplied!\n");
+      ret=KEYSERVER_SCHEME_NOT_FOUND;
+      goto fail;
+    }
+
+  if(strcasecmp(opt->scheme,"ldaps")==0)
+    {
+      port=636;
+      use_ssl=1;
+    }
+
+  if(opt->port)
+    port=atoi(opt->port);
+
+  if(!opt->host)
+    {
+      fprintf(console,"gpgkeys: no keyserver host provided\n");
+      goto fail;
+    }
+
+  if(opt->timeout && register_timeout()==-1)
     {
       fprintf(console,"gpgkeys: unable to register timeout handler\n");
       return KEYSERVER_INTERNAL_ERROR;
     }
 
+#if defined(LDAP_OPT_X_TLS_CACERTFILE) && defined(HAVE_LDAP_SET_OPTION)
+
+  if(opt->ca_cert_file)
+    {
+      err=ldap_set_option(NULL,LDAP_OPT_X_TLS_CACERTFILE,opt->ca_cert_file);
+      if(err!=LDAP_SUCCESS)
+	{
+	  fprintf(console,"gpgkeys: unable to set ca-cert-file: %s\n",
+		  ldap_err2string(err));
+	  ret=KEYSERVER_INTERNAL_ERROR;
+	  goto fail;
+	}
+    }
+#endif /* LDAP_OPT_X_TLS_CACERTFILE && HAVE_LDAP_SET_OPTION */
+
   /* SSL trumps TLS */
   if(use_ssl)
     use_tls=0;
@@ -1784,9 +1991,10 @@ main(int argc,char *argv[])
   /* If it's a GET or a SEARCH, the next thing to come in is the
      keyids.  If it's a SEND, then there are no keyids. */
 
-  if(action==SEND)
+  if(opt->action==KS_SEND)
     while(fgets(line,MAX_LINE,input)!=NULL && line[0]!='\n');
-  else if(action==GET || action==SEARCH)
+  else if(opt->action==KS_GET
+	  || opt->action==KS_GETNAME || opt->action==KS_SEARCH)
     {
       for(;;)
 	{
@@ -1837,24 +2045,23 @@ main(int argc,char *argv[])
   fprintf(output,"VERSION %d\n",KEYSERVER_PROTO_VERSION);
   fprintf(output,"PROGRAM %s\n\n",VERSION);
 
-  if(verbose>1)
+  if(opt->verbose>1)
     {
-      fprintf(console,"Host:\t\t%s\n",host);
+      fprintf(console,"Host:\t\t%s\n",opt->host);
       if(port)
 	fprintf(console,"Port:\t\t%d\n",port);
-      fprintf(console,"Command:\t%s\n",action==GET?"GET":
-	      action==SEND?"SEND":"SEARCH");
+      fprintf(console,"Command:\t%s\n",ks_action_to_string(opt->action));
     }
 
-  if(debug)
+  if(opt->debug)
     {
 #if defined(LDAP_OPT_DEBUG_LEVEL) && defined(HAVE_LDAP_SET_OPTION)
-      err=ldap_set_option(NULL,LDAP_OPT_DEBUG_LEVEL,&debug);
+      err=ldap_set_option(NULL,LDAP_OPT_DEBUG_LEVEL,&opt->debug);
       if(err!=LDAP_SUCCESS)
 	fprintf(console,"gpgkeys: unable to set debug mode: %s\n",
 		ldap_err2string(err));
       else
-	fprintf(console,"gpgkeys: debug level %d\n",debug);
+	fprintf(console,"gpgkeys: debug level %d\n",opt->debug);
 #else
       fprintf(console,"gpgkeys: not built with debugging support\n");
 #endif
@@ -1862,35 +2069,49 @@ main(int argc,char *argv[])
 
   /* We have a timeout set for the setup stuff since it could time out
      as well. */
-  set_timeout(timeout);
+  set_timeout(opt->timeout);
 
   /* Note that this tries all A records on a given host (or at least,
      OpenLDAP does). */
-  ldap=ldap_init(host,port);
+  ldap=ldap_init(opt->host,port);
   if(ldap==NULL)
     {
       fprintf(console,"gpgkeys: internal LDAP init error: %s\n",
 	      strerror(errno));
-      fail_all(keylist,action,KEYSERVER_INTERNAL_ERROR);
+      fail_all(keylist,KEYSERVER_INTERNAL_ERROR);
       goto fail;
     }
 
   if(use_ssl)
     {
-#if defined(LDAP_OPT_X_TLS_HARD) && defined(HAVE_LDAP_SET_OPTION)
+#if defined(LDAP_OPT_X_TLS) && defined(HAVE_LDAP_SET_OPTION)
       int ssl=LDAP_OPT_X_TLS_HARD;
+
       err=ldap_set_option(ldap,LDAP_OPT_X_TLS,&ssl);
       if(err!=LDAP_SUCCESS)
 	{
 	  fprintf(console,"gpgkeys: unable to make SSL connection: %s\n",
 		  ldap_err2string(err));
-	  fail_all(keylist,action,ldap_err_to_gpg_err(err));
+	  fail_all(keylist,ldap_err_to_gpg_err(err));
+	  goto fail;
+	}
+
+      if(!opt->flags.check_cert)
+	ssl=LDAP_OPT_X_TLS_NEVER;
+
+      err=ldap_set_option(NULL,LDAP_OPT_X_TLS_REQUIRE_CERT,&ssl);
+      if(err!=LDAP_SUCCESS)
+	{
+	  fprintf(console,
+		  "gpgkeys: unable to set certificate validation: %s\n",
+		  ldap_err2string(err));
+	  fail_all(keylist,ldap_err_to_gpg_err(err));
 	  goto fail;
 	}
 #else
       fprintf(console,"gpgkeys: unable to make SSL connection: %s\n",
 	      "not built with LDAPS support");
-      fail_all(keylist,action,KEYSERVER_INTERNAL_ERROR);
+      fail_all(keylist,KEYSERVER_INTERNAL_ERROR);
       goto fail;
 #endif
     }
@@ -1900,7 +2121,7 @@ main(int argc,char *argv[])
       {
 	fprintf(console,"gpgkeys: unable to retrieve LDAP base: %s\n",
 		err?ldap_err2string(err):"not found");
-	fail_all(keylist,action,ldap_err_to_gpg_err(err));
+	fail_all(keylist,ldap_err_to_gpg_err(err));
 	goto fail;
       }
 
@@ -1915,7 +2136,7 @@ main(int argc,char *argv[])
 		    "not supported by the NAI LDAP keyserver");
 	  if(use_tls==3)
 	    {
-	      fail_all(keylist,action,KEYSERVER_INTERNAL_ERROR);
+	      fail_all(keylist,KEYSERVER_INTERNAL_ERROR);
 	      goto fail;
 	    }
       	}
@@ -1924,25 +2145,36 @@ main(int argc,char *argv[])
 #if defined(HAVE_LDAP_START_TLS_S) && defined(HAVE_LDAP_SET_OPTION)
 	  int ver=LDAP_VERSION3;
 
-	  err=LDAP_SUCCESS;
-
 	  err=ldap_set_option(ldap,LDAP_OPT_PROTOCOL_VERSION,&ver);
+
+#ifdef LDAP_OPT_X_TLS
+	  if(err==LDAP_SUCCESS)
+	    {
+	      if(opt->flags.check_cert)
+		ver=LDAP_OPT_X_TLS_HARD;
+	      else
+		ver=LDAP_OPT_X_TLS_NEVER;
+
+	      err=ldap_set_option(ldap,LDAP_OPT_X_TLS_REQUIRE_CERT,&ver);
+	    }
+#endif
+
 	  if(err==LDAP_SUCCESS)
 	    err=ldap_start_tls_s(ldap,NULL,NULL);
 
 	  if(err!=LDAP_SUCCESS)
 	    {
-	      if(use_tls==2 || verbose>2)
+	      if(use_tls>=2 || opt->verbose>2)
 		fprintf(console,"gpgkeys: unable to start TLS: %s\n",
 			ldap_err2string(err));
 	      /* Are we forcing it? */
 	      if(use_tls==3)
 		{
-		  fail_all(keylist,action,ldap_err_to_gpg_err(err));
+		  fail_all(keylist,ldap_err_to_gpg_err(err));
 		  goto fail;
 		}
 	    }
-	  else if(err==LDAP_SUCCESS && verbose>1)
+	  else if(opt->verbose>1)
 	    fprintf(console,"gpgkeys: TLS started successfully.\n");
 #else
 	  if(use_tls>=2)
@@ -1950,120 +2182,144 @@ main(int argc,char *argv[])
 		    "not built with TLS support");
 	  if(use_tls==3)
 	    {
-	      fail_all(keylist,action,KEYSERVER_INTERNAL_ERROR);
+	      fail_all(keylist,KEYSERVER_INTERNAL_ERROR);
 	      goto fail;
 	    }
 #endif
 	}
     }
 
-#if 0
-  /* The LDAP keyserver doesn't require this, but it might be useful
-     if someone stores keys on a V2 LDAP server somewhere.  (V3
-     doesn't require a bind).  Leave this out for now since it is not
-     clear if anyone server we're likely to use really cares, plus
-     there are some servers that don't allow it. */
+  /* By default we don't bind as there is usually no need to.  For
+     cases where the server needs some authentication, the user can
+     use binddn and bindpw for auth. */
 
-  err=ldap_simple_bind_s(ldap,NULL,NULL);
-  if(err!=0)
+  if(binddn)
     {
-      fprintf(console,"gpgkeys: internal LDAP bind error: %s\n",
-	      ldap_err2string(err));
-      fail_all(keylist,action,ldap_err_to_gpg_err(err));
-      goto fail;
-    }
-  else
-    bound=1;
+#ifdef HAVE_LDAP_SET_OPTION
+      int ver=LDAP_VERSION3;
+
+      err=ldap_set_option(ldap,LDAP_OPT_PROTOCOL_VERSION,&ver);
+      if(err!=LDAP_SUCCESS)
+	{
+	  fprintf(console,"gpgkeys: unable to go to LDAP 3: %s\n",
+		  ldap_err2string(err));
+	  fail_all(keylist,ldap_err_to_gpg_err(err));
+	  goto fail;
+	}
 #endif
 
-  switch(action)
+      if(opt->verbose>2)
+	fprintf(console,"gpgkeys: LDAP bind to %s, pw %s\n",binddn,
+		bindpw?">not shown<":">none<");
+      err=ldap_simple_bind_s(ldap,binddn,bindpw);
+      if(err!=LDAP_SUCCESS)
+	{
+	  fprintf(console,"gpgkeys: internal LDAP bind error: %s\n",
+		  ldap_err2string(err));
+	  fail_all(keylist,ldap_err_to_gpg_err(err));
+	  goto fail;
+	}
+      else
+	bound=1;
+    }
+
+  if(opt->action==KS_GET)
     {
-    case GET:
       keyptr=keylist;
 
       while(keyptr!=NULL)
 	{
-	  set_timeout(timeout);
+	  set_timeout(opt->timeout);
 
 	  if(get_key(keyptr->str)!=KEYSERVER_OK)
 	    failed++;
 
 	  keyptr=keyptr->next;
 	}
-      break;
-
-    case SEND:
-      {
-	int eof=0;
-
-	do
-	  {
-	    set_timeout(timeout);
-
-	    if(real_ldap)
-	      {
-		if(send_key(&eof)!=KEYSERVER_OK)
-		  failed++;
-	      }
-	    else
-	      {
-		if(send_key_keyserver(&eof)!=KEYSERVER_OK)
-		  failed++;
-	      }
-	  }
-	while(!eof);
-      }
-      break;
-
-    case SEARCH:
-      {
-	char *searchkey=NULL;
-	int len=0;
-
-	set_timeout(timeout);
-
-	/* To search, we stick a * in between each key to search for.
-	   This means that if the user enters words, they'll get
-	   "enters*words".  If the user "enters words", they'll get
-	   "enters words" */
-
-	keyptr=keylist;
-	while(keyptr!=NULL)
-	  {
-	    len+=strlen(keyptr->str)+1;
-	    keyptr=keyptr->next;
-	  }
-
-	searchkey=malloc(len+1);
-	if(searchkey==NULL)
-	  {
-	    ret=KEYSERVER_NO_MEMORY;
-	    fail_all(keylist,action,KEYSERVER_NO_MEMORY);
-	    goto fail;
-	  }
-
-	searchkey[0]='\0';
-
-	keyptr=keylist;
-	while(keyptr!=NULL)
-	  {
-	    strcat(searchkey,keyptr->str);
-	    strcat(searchkey,"*");
-	    keyptr=keyptr->next;
-	  }
-
-	/* Nail that last "*" */
-	if(*searchkey)
-	  searchkey[strlen(searchkey)-1]='\0';
-
-	if(search_key(searchkey)!=KEYSERVER_OK)
-	  failed++;
-
-	free(searchkey);
-      }
-
-      break;
     }
+  else if(opt->action==KS_GETNAME)
+    {
+      keyptr=keylist;
+
+      while(keyptr!=NULL)
+	{
+	  set_timeout(opt->timeout);
+
+	  if(get_name(keyptr->str)!=KEYSERVER_OK)
+	    failed++;
+
+	  keyptr=keyptr->next;
+	}
+    }
+  else if(opt->action==KS_SEND)
+    {
+      int eof=0;
+
+      do
+	{
+	  set_timeout(opt->timeout);
+
+	  if(real_ldap)
+	    {
+	      if(send_key(&eof)!=KEYSERVER_OK)
+		failed++;
+	    }
+	  else
+	    {
+	      if(send_key_keyserver(&eof)!=KEYSERVER_OK)
+		failed++;
+	    }
+	}
+      while(!eof);
+    }
+  else if(opt->action==KS_SEARCH)
+    {
+      char *searchkey=NULL;
+      int len=0;
+
+      set_timeout(opt->timeout);
+
+      /* To search, we stick a * in between each key to search for.
+	 This means that if the user enters words, they'll get
+	 "enters*words".  If the user "enters words", they'll get
+	 "enters words" */
+
+      keyptr=keylist;
+      while(keyptr!=NULL)
+	{
+	  len+=strlen(keyptr->str)+1;
+	  keyptr=keyptr->next;
+	}
+
+      searchkey=malloc(len+1);
+      if(searchkey==NULL)
+	{
+	  ret=KEYSERVER_NO_MEMORY;
+	  fail_all(keylist,KEYSERVER_NO_MEMORY);
+	  goto fail;
+	}
+
+      searchkey[0]='\0';
+
+      keyptr=keylist;
+      while(keyptr!=NULL)
+	{
+	  strcat(searchkey,keyptr->str);
+	  strcat(searchkey,"*");
+	  keyptr=keyptr->next;
+	}
+
+      /* Nail that last "*" */
+      if(*searchkey)
+	searchkey[strlen(searchkey)-1]='\0';
+
+      if(search_key(searchkey)!=KEYSERVER_OK)
+	failed++;
+
+      free(searchkey);
+    }
+  else
+    BUG();
 
   if(!failed)
     ret=KEYSERVER_OK;
@@ -2083,6 +2339,8 @@ main(int argc,char *argv[])
   if(output!=stdout)
     fclose(output);
 
+  free_ks_options(opt);
+
   if(ldap!=NULL && bound)
     ldap_unbind_s(ldap);
 
