@@ -63,15 +63,17 @@ if test "$1" = "--build-w32"; then
           echo "We need at least version 0.3 of MingW32/CPD" >&2
           exit 1
        fi
-       crossbindir=`mingw32 --install-dir`/bin
-       # Old autoconf version required us to setup the environment
-       # with the proper tool names.
-       CC=`mingw32 --get-path gcc`
-       CPP=`mingw32 --get-path cpp`
-       AR=`mingw32 --get-path ar`
-       RANLIB=`mingw32 --get-path ranlib`
-       export CC CPP AR RANLIB 
-       conf_CC=""
+       echo "MingW32/CPD is no longer supported" >&2
+       exit 1
+       # crossbindir=`mingw32 --install-dir`/bin
+       # # Old autoconf version required us to setup the environment
+       # # with the proper tool names.
+       # CC=`mingw32 --get-path gcc`
+       # CPP=`mingw32 --get-path cpp`
+       # AR=`mingw32 --get-path ar`
+       # RANLIB=`mingw32 --get-path ranlib`
+       # export CC CPP AR RANLIB 
+       # conf_CC=""
     fi
    
     if [ -f "$tsdir/config.log" ]; then
@@ -99,11 +101,6 @@ if test "$1" = "--build-w32"; then
 
     $tsdir/configure ${conf_CC} --build=${build} --host=${host} \
                 ${disable_foo_tests}  $*
-
-    # Ugly hack to overcome a gettext problem.  Someone should look into
-    # gettext to figure out why the po directory is not ignored as it used
-    # to be.
-    [ $? = 0 ] && touch $tsdir/po/all
     exit $?
 fi
 
@@ -273,8 +270,8 @@ echo "Running aclocal -I m4 ${ACLOCAL_FLAGS:+$ACLOCAL_FLAGS }..."
 $ACLOCAL -I m4 $ACLOCAL_FLAGS
 echo "Running autoheader..."
 $AUTOHEADER
-echo "Running automake --gnu ..."
-$AUTOMAKE --gnu;
+echo "Running automake --gnu --add-missing..."
+$AUTOMAKE --gnu --add-missing;
 echo "Running autoconf..."
 $AUTOCONF
 
