@@ -43,6 +43,12 @@
 #include "util.h"
 #include "i18n.h"
 
+/* MinGW doesn't seem to prototype getpagesize, though it does have
+   it. */
+#if !HAVE_DECL_GETPAGESIZE
+int getpagesize(void);
+#endif
+
 #if defined(MAP_ANON) && !defined(MAP_ANONYMOUS)
 #define MAP_ANONYMOUS MAP_ANON
 #endif
@@ -394,7 +400,7 @@ secmem_malloc( size_t size )
 
 
 void *
-secmem_realloc( void *p, size_t newsize )
+secmexrealloc( void *p, size_t newsize )
 {
     MEMBLOCK *mb;
     size_t size;
@@ -403,7 +409,7 @@ secmem_realloc( void *p, size_t newsize )
     mb = (MEMBLOCK*)((char*)p - ((size_t) &((MEMBLOCK*)0)->u.aligned.c));
     size = mb->size;
     if (size < sizeof(MEMBLOCK))
-      log_bug ("secure memory corrupted at block %p\n", mb);
+      log_bug ("secure memory corrupted at block %p\n", (void *)mb);
     size -= ((size_t) &((MEMBLOCK*)0)->u.aligned.c);
 
     if( newsize <= size )
